-- (Offline) Outliers Clustering
module OC where
import Metric
import Data.List (maximumBy)
import Data.Function (on)

-- Offline algorithm for k-center clustering with outliers.
-- Finds a k-center clustering of radius 4r with centers from pts that contains
-- at least as many pts as any k-center clustering of radius 2r with
-- unrestricted centers.
tryOutliersCluster r k pts =
	-- If we're out of centers or points, we're done.
	if (k == 0) || (null pts) then []
	else let nPtsWithin2r c = length (filter (\p -> dist c p <= 2*r) pts) in
	     -- Choose a center greedily.
	     let newCtr = maximumBy (compare `on` nPtsWithin2r) pts in
	     -- Remove points within 4r.
	     let remainingPts = filter (\p -> dist newCtr p > 4*r) pts in
	     -- Cluster the remaining points.
	     newCtr : tryOutliersCluster r (k - 1) remainingPts

-- How many pts do the ctrs cover at radius r?
-- Use to test whether tryOutliersCluster did as well as you hoped.
nPtsCovered ctrs pts r = length (filter (\p -> any (\c -> dist c p <= r) ctrs) pts)
